package WebService::Braintree::ErrorCodes::MerchantAccount::Funding;
$WebService::Braintree::ErrorCodes::MerchantAccount::Funding::VERSION = '0.3';
use strict;

use constant RoutingNumberIsRequired                          => "82640";
use constant AccountNumberIsRequired                          => "82641";
use constant RoutingNumberIsInvalid                           => "82649";
use constant AccountNumberIsInvalid                           => "82671";
use constant DestinationIsRequired                            => "82678";
use constant DestinationIsInvalid                             => "82679";
use constant EmailIsRequired                                  => "82680";
use constant EmailIsInvalid                                   => "82681";
use constant MobilePhoneIsRequired                            => "82682";
use constant MobilePhoneIsInvalid                             => "82683";

1;
