package WebService::Braintree::ErrorCodes::Descriptor;
$WebService::Braintree::ErrorCodes::Descriptor::VERSION = '0.3';
use strict;

use constant DynamicDescriptorsDisabled        => "92203";
use constant InternationalPhoneFormatIsInvalid => "92205";
use constant InternationalNameFormatIsInvalid  => "92204";
use constant NameFormatIsInvalid               => "92201";
use constant PhoneFormatIsInvalid              => "92202";
use constant UrlFormatIsInvalid                => "92206";

1;
