package WebService::Braintree::Transaction::CreatedUsing;
$WebService::Braintree::Transaction::CreatedUsing::VERSION = '0.3';

use constant Token => "token";
use constant FullInformation => "full_information";

1;
