package WebService::Braintree::ErrorCodes::SettlementBatchSummary;
$WebService::Braintree::ErrorCodes::SettlementBatchSummary::VERSION = '0.3';
use strict;

use constant CustomFieldIsInvalid     => "82303";
use constant SettlementDateIsInvalid  => "82302";
use constant SettlementDateIsRequired => "82301";

1;
