package WebService::Braintree::ErrorCodes;
$WebService::Braintree::ErrorCodes::VERSION = '0.3';
use strict;

use WebService::Braintree::ErrorCodes::Address;
use WebService::Braintree::ErrorCodes::ApplePay;
use WebService::Braintree::ErrorCodes::CreditCard;
use WebService::Braintree::ErrorCodes::CreditCard::Options;
use WebService::Braintree::ErrorCodes::Customer;
use WebService::Braintree::ErrorCodes::Descriptor;
use WebService::Braintree::ErrorCodes::MerchantAccount;
use WebService::Braintree::ErrorCodes::MerchantAccount::ApplicantDetails;
use WebService::Braintree::ErrorCodes::MerchantAccount::ApplicantDetails::Address;
use WebService::Braintree::ErrorCodes::AltPay;
use WebService::Braintree::ErrorCodes::SettlementBatchSummary;
use WebService::Braintree::ErrorCodes::Subscription;
use WebService::Braintree::ErrorCodes::Subscription::Modification;
use WebService::Braintree::ErrorCodes::Transaction;
use WebService::Braintree::ErrorCodes::Transaction::Options;

1;
