package WebService::Braintree::ErrorCodes::IndustryType;
$WebService::Braintree::ErrorCodes::IndustryType::VERSION = '0.3';
use strict;

use constant CheckInDateIsInvalid               => "93404";
use constant CheckOutDateIsInvalid              => "93405";
use constant CheckOutDateMustFollowCheckInDate  => "93406";
use constant EmptyData                          => "93402";
use constant FolioNumberIsInvalid               => "93403";
use constant IndustryTypeIsInvalid              => "93401";
use constant UnknownDataField                   => "93407";

1;
