package WebService::Braintree::CreditCard::Location;
$WebService::Braintree::CreditCard::Location::VERSION = '0.3';
use strict;

use constant International => "international";
use constant US => "us";

1;
