package WebService::Braintree::CreditCard::CountryOfIssuance;
$WebService::Braintree::CreditCard::CountryOfIssuance::VERSION = '0.3';
use strict;

use constant Yes => "Yes";
use constant No => "No";
use constant Unknown => "Unknown";

1;
