package WebService::Braintree::Dispute::Status;
$WebService::Braintree::Dispute::Status::VERSION = '0.3';
use strict;

use constant Open => 'open';
use constant Won => 'won';
use constant Lost => 'lost';

1;
