package WebService::Braintree::ErrorCodes::CreditCard::Options;
$WebService::Braintree::ErrorCodes::CreditCard::Options::VERSION = '0.3';
use strict;

use constant UpdateExistingTokenIsInvalid           => "91723";
use constant UseBillingForShippingDisabled          => "91572";
use constant VerificationMerchantAccountIdIsInvalid => "91728";

1;
